@@ -1,10 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
-#game site:
-#http://wap.jue-huo.com/app/html/game/1to50/1to50.html
-
-import sys,dlib,numpy
-
+import sys,dlib
+import numpy as np
 #pip install scikit-image
 from skimage import io
 import cv2
@@ -17,19 +14,20 @@ from os.path import dirname
 import requests
 import bz2#bz2file pypi
 import zipfile
+
 def loadDBFromFeatureFiles(folder):
     files = os.listdir(folder)
     with open(folder+"\\" + "IDs.txt",encoding='utf8') as ff:
         IDs = ff.readlines()
     for ID in IDs:
         print("load "+ ID.strip() +".npy")
-        DB[ID.strip()]=numpy.load(folder+"\\"+ID.strip()+'.npy')
+        DB[ID.strip()]=np.load(folder+"\\"+ID.strip()+'.npy')
             
             
 def saveDB(folder):
     _createFolder(folder)
     for key in DB.keys():
-        numpy.save(folder+"\\"+key.strip(),DB[key.strip()])
+        np.save(folder+"\\"+key.strip(),DB[key.strip()])
     with open(folder+'\\'+'IDs.txt','w',encoding='utf8') as fkey:        
         for key in DB.keys():
             fkey.write(key.strip()+"\n")        
@@ -55,7 +53,7 @@ def _save_response_content(response, destination):
             if chunk: # filter out keep-alive new chunks
                 f.write(chunk)
 #==============================
-def _download_url_response(path,url):
+def download_url_response(path,url):
     r = requests.get(url, stream = True)
     with open(path, 'wb') as f:
         i=0
@@ -91,22 +89,26 @@ def downloadFileInNeed(filename):
     http://dlib.net/files
     filename: for example ==>"shape_predictor_68_face_landmarks.dat"
     '''
-    if not os.path.isfile(filename):
-        print("載入"+filename+"...可能需要一點時間") 
+    if os.path.isfile(filename):return
+    print("載入"+filename+"...可能需要一點時間") 
+    if filename.find('.ttf')>=0:
+        download_url_response("kaiu.ttf","http://sf1.loxa.edu.tw/104736/Download/kaiu.ttf")
+    else:
         url="http://dlib.net/files/"+filename+".bz2"
-        _download_url_response(filename+".bz2",url)
+        download_url_response(filename+".bz2",url)
         _bz2Decode(filename+".bz2")
         os.remove(filename+".bz2")
         print("ok")
         
 downloadFileInNeed("shape_predictor_68_face_landmarks.dat")  
 downloadFileInNeed("dlib_face_recognition_resnet_model_v1.dat")
-    
+downloadFileInNeed("kaiu.ttf")
+
 Package_Dir = dirname(__file__)
 #載入字型
 _FONT = ImageFont.truetype("kaiu.ttf",20,index=0)
 # 載入人臉檢測器
-_detector = dlib.get_frontal_face_detector()
+detector = dlib.get_frontal_face_detector()
 # 載入人臉特徵點檢測器
 _sp = dlib.shape_predictor("shape_predictor_68_face_landmarks.dat")
 # 載入人臉辨識檢測器
@@ -129,25 +131,19 @@ def getFeatureVector(img,rect=None):
     '''
     if not rect:
         try:
-            rect=_detector(img, 1)[0] #如果沒有傳入rect取第一個檢測到的臉區域
+            rect=detector(img, 1)[0] #如果沒有傳入rect取第一個檢測到的臉區域
         except:return []
     shape = _sp(img,rect) #找出特徵點位置
     #由圖片中特徵點位置(shape)擷取出特徵向量(128維特徵向量)
     face_descriptor = _facerec.compute_face_descriptor(img, shape)
     # 轉換numpy array格式
-    return numpy.array(face_descriptor)     
+    return np.array(face_descriptor)     
 
 def getFaceRects(img):
     try:
-        return _detector(img, 1)
+        return detector(img, 1)
     except:return []
     
-def loadImg(fname):
-    '''
-    loadImg(fname)
-    load image from file : return a opencv image object
-    '''
-    return io.imread(fname)
 
 def loadDB(db=DB,folder='train',isClearDB=True):
     '''
@@ -156,7 +152,6 @@ def loadDB(db=DB,folder='train',isClearDB=True):
     DB is a default dictionary object for db
     default folder should be "train"
     the files should be like:
-    
     John.jpg
     KY.jpg
     Marry.jpg
@@ -166,34 +161,14 @@ def loadDB(db=DB,folder='train',isClearDB=True):
     '''
     if isClearDB:db.clear()
     for f in os.listdir(folder):
-        path = folder+"\\"+f
+        path = f"{folder}/{f}"
         if os.path.isfile(path):
-            img = io.imread(path)
+            img=io.imread(path)#cv2.imread(path)==>opencv的這個函式在windows裡面讀中文檔名有問題
             FV = getFeatureVector(img)
             if len(FV)>0:
-                db[os.path.splitext(f)[0]]=getFeatureVector(img)
+                db[f[:f.rfind('.')]]=getFeatureVector(img)
                 print(f+" feature loaded")
             else:print(f+" fail to detect face!!")
-def loadFeatureFromPic(imagePath):
-    '''
-    loadFeatureFromPic(imagePath)
-    load image from imagePath and return the Feture Vector (rank=128)
-    '''
-    img = io.imread(imagePath)
-    return getFeatureVector(img)
-
-def loadPicRawFeature(fname):
-    '''
-    loadPicRawFeature(fname)
-    load image and return nomalized(divide by 255) raw feature of RGB value
-    RGB RGB RGB ...
-    '''
-    try:
-        img = io.imread(fname)
-        #img.tofile('test.txt',',')
-        return img.flatten()/255.0
-    except Exception:print(Exception.args)
-    return None
 
 def predictFromDB(VTest,db=DB):
     '''
@@ -204,7 +179,7 @@ def predictFromDB(VTest,db=DB):
     minD = sys.float_info.max
     minK=''
     for k in db:
-        dist=numpy.linalg.norm(VTest-db[k])
+        dist=np.linalg.norm(VTest-db[k])
         if dist<minD:
             minK=k
             minD=dist
@@ -215,9 +190,9 @@ def dist(V1,V2):
     '''
     dist(V1,V2)
     return distance between V1,V2
-    V1,V2:numpy.array
+    V1,V2:np.array
     '''
-    return numpy.linalg.norm(V1-V2)
+    return np.linalg.norm(V1-V2)
 
 def addText2Img_cv2(img_cv2,text,font=_FONT,position=(20,20),fill=(255,0,0)):
     '''
@@ -229,7 +204,7 @@ def addText2Img_cv2(img_cv2,text,font=_FONT,position=(20,20),fill=(255,0,0)):
     img_PIL = Image.fromarray(cv2.cvtColor(img_cv2,cv2.COLOR_BGR2RGB))#cv2.COLOR_BGR2RGB cv2.COLOR_RGB2BGR
     draw = ImageDraw.Draw(img_PIL)
     draw.text(position, text, font=font, fill=fill)
-    ret = cv2.cvtColor(numpy.asarray(img_PIL),cv2.COLOR_RGB2BGR)# 转换回OpenCV格式
+    ret = cv2.cvtColor(np.asarray(img_PIL),cv2.COLOR_RGB2BGR)# 转换回OpenCV格式
     img_PIL.close()
     return ret
 
@@ -242,9 +217,9 @@ def captureImageFromCam(camSerial=None):
     cap = None
     if not camSerial:
         for i in range(5):#assume there are less than 5 webcam attached to the platform
-            cap = cv2.VideoCapture(i)
+            cap = cv2.VideoCapture(i,cv2.CAP_DSHOW)
             if cap.isOpened():break
-    else: cap = cv2.VideoCapture(camSerial)
+    else: cap = cv2.VideoCapture(camSerial,cv2.CAP_DSHOW)
     if not cap:return None
     
     # 載入人臉檢測器
@@ -268,9 +243,9 @@ def captureImageFromCamAndSave(tag,folder="train",camSerial=None):
     cap = None
     if not camSerial:
         for i in range(5):#assume there are less than 5 webcam attached to the platform
-            cap = cv2.VideoCapture(i)
+            cap = cv2.VideoCapture(i,cv2.CAP_DSHOW)
             if cap.isOpened():break
-    else: cap = cv2.VideoCapture(camSerial)
+    else: cap = cv2.VideoCapture(camSerial,cv2.CAP_DSHOW)
     if not cap:return None
     
     cv2.startWindowThread()
@@ -285,7 +260,7 @@ def captureImageFromCamAndSave(tag,folder="train",camSerial=None):
             # 顯示圖片
             if ret: 
                 try:
-                    rect=_detector(frame, 1)[0] #取第一個檢測到的臉區域# IndexError
+                    rect=detector(frame, 1)[0] #取第一個檢測到的臉區域# IndexError
                     if cv2.waitKey(1) & 0xFF == ord('p'):
                         im=cv2.cvtColor(frame, cv2.COLOR_RGB2BGR)
                         
@@ -309,7 +284,7 @@ def predictImage(frame,db=DB,showResult=True):
     '''
     predictImage(frame,db=DB,showResult=True)
     '''
-    rects=_detector(frame, 1)
+    rects=detector(frame, 1)
     Tags = [];dists = []
     for rect in rects:
         try:
@@ -338,11 +313,11 @@ def predictCam(camSerial=None,skipFranmes=1,outputFileName="",db=DB):
     cap = None
     if not camSerial:
         for i in range(5):
-            cap = cv2.VideoCapture(i)
+            cap = cv2.VideoCapture(i,cv2.CAP_DSHOW)
             if cap.isOpened():break
-    else: cap = cv2.VideoCapture(camSerial)
+    else: cap = cv2.VideoCapture(camSerial,cv2.CAP_DSHOW)
     cv2.startWindowThread()
-    #_detector = dlib.get_frontal_face_detector()
+    #detector = dlib.get_frontal_face_detector()
     count = 0
     while(True):    
         if not cap.isOpened():break
@@ -380,7 +355,7 @@ def predictVedio(vedioPath,skipFranmes=10,db=DB):
         count+=1
         if count%skipFranmes!=0 :continue
         try:
-            rects=_detector(frame, 1)
+            rects=detector(frame, 1)
             for rect in rects:
                 V=getFeatureVector(frame,rect)
                 Tag,dist=predictFromDB(V,db) # predict the target Tag
@@ -394,48 +369,10 @@ def predictVedio(vedioPath,skipFranmes=10,db=DB):
     cap.release()
     cv2.destroyAllWindows()
     
-from skimage import transform
-import matplotlib.pyplot as plt
+#from skimage import transform
+#import matplotlib.pyplot as plt
 #from skimage.util import crop
 if __name__ == "__main__":
-    #captureImageFromCamAndSave('李國源',camSerial=1)
-    '''
-    img = loadImg('train/林志玲.jpg')
-    rect = getFaceRect(img)
-    print(rect)
-    faceImg = img[rect.top():rect.bottom(),rect.left():rect.right()]
-    dst=transform.downscale_local_mean(faceImg,(15,15,1))
-    dst=transform.resize(dst,(200,200),anti_aliasing=False)
-    print(numpy.shape(dst))
-    fig, axes = plt.subplots(nrows=1, ncols=2)
-    ax = axes.ravel()
-    ax[0].imshow(dst)
-    ax[1].imshow(faceImg)
-    plt.tight_layout()
-    plt.show()
-    io.imsave('test.png',dst)
-    io.imsave('test1.png',faceImg)
-    '''
-    
-    '''
-    #download the samples to 'train' folder
-    downloadImageSamples()
-    # 比對目前webcam擷取到的人臉和指定影像檔案並計算它們之間的距離
-    im = captureImageFromCam()
-    VTest = getFeatureVector(im)
-    Vtrain = loadFeatureFromPic('train\\李國源.jpg')
-    print(dist(VTest,Vtrain))
-    
-    #載入train資料夾中所有jpg檔之特徵及tag
-    #並直接預測目前webcam擷取到的人臉對應的TAG
-    loadDB(folder='train')
-    result = predictFromDB(VTest)
-    print(result)
-    '''
-    #vedio Demo
-    #loadDB(folder='train')
-    #
-    #saveDB('features')
-    #loadDBFromFeatureFiles("features")
-    #print(DB['四葉草'])
-    #predictCam(outputFileName="result.txt")
\ No newline at end of file
+    pass
+
+
